use super::{
    get_player_item_or_fail, get_user_or_fail, user_error, UResult, UserVerb, UserVerbRef,
    VerbContext,
};
use crate::models::corp::CorpCommType;
use ansi::{ansi, ignore_special_characters};
use async_trait::async_trait;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        remaining: &str,
    ) -> UResult<()> {
        let user = get_user_or_fail(ctx)?;

        let (corp_id, corp, msg) = if remaining.starts_with("@") {
            match remaining[1..].split_once(" ") {
                None => user_error(
                    "Usage: c message (lowest ordered corp) or c @corpname message".to_owned(),
                )?,
                Some((corpname, msg)) => {
                    let (corp_id, corp, _) = match ctx
                        .trans
                        .match_user_corp_by_name(corpname.trim(), &user.username)
                        .await?
                    {
                        None => {
                            user_error("You don't seem to belong to a matching corp!".to_owned())?
                        }
                        Some(c) => c,
                    };
                    (corp_id, corp, msg.trim())
                }
            }
        } else {
            let (corp_id, corp) = match ctx.trans.get_default_corp_for_user(&user.username).await? {
                None => user_error("You're not a member of any corps.".to_owned())?,
                Some(v) => v,
            };
            (corp_id, corp, remaining.trim())
        };

        let msg = ignore_special_characters(msg);

        if msg.trim() == "" {
            user_error("Message required.".to_owned())?;
        }

        let player = get_player_item_or_fail(ctx).await?;
        ctx.trans
            .broadcast_to_corp(
                &corp_id,
                &CorpCommType::Chat,
                Some(&user.username),
                &format!(
                    ansi!("<yellow>{} (to {}): <reset><bold>\"{}\"<reset>\n"),
                    &player.display_for_sentence(1, true),
                    &corp.name,
                    &msg
                ),
            )
            .await?;

        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
