use crate::{models::effect::EffectType, services::combat::switch_to_feint};

use super::{get_player_item_or_fail, user_error, UResult, UserVerb, UserVerbRef, VerbContext};
use async_trait::async_trait;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        _remaining: &str,
    ) -> UResult<()> {
        let player_item = get_player_item_or_fail(ctx).await?;
        if player_item.death_data.is_some() {
            user_error("Feint while dead? You can't even do a regular attack.".to_owned())?;
        }

        if player_item
            .active_effects
            .iter()
            .any(|v| v.0 == EffectType::Stunned)
        {
            user_error(
                "You stay still like a stunned mullet, unable to gain the composure to feint."
                    .to_owned(),
            )?;
        }

        switch_to_feint(ctx, &player_item).await?;

        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
