use super::{
    get_player_item_or_fail, get_user_or_fail, user_error, UResult, UserVerb, UserVerbRef,
    VerbContext,
};
use crate::{language, static_content::possession_type::possession_data, static_content::room};
use ansi::ansi;
use async_trait::async_trait;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        _remaining: &str,
    ) -> UResult<()> {
        let player_item = get_player_item_or_fail(ctx).await?;
        let user = get_user_or_fail(ctx)?;

        if player_item.death_data.is_some() {
            user_error(
                "Nobody seems to offer you any prices... possibly because you're dead.".to_owned(),
            )?
        }
        let (heretype, herecode) = player_item
            .location
            .split_once("/")
            .unwrap_or(("room", "repro_xv_chargen"));
        if heretype != "room" {
            user_error("Can't list stock because you're not in a shop.".to_owned())?;
        }
        let room = match room::room_map_by_code().get(herecode) {
            None => user_error("Can't find that shop.".to_owned())?,
            Some(r) => r,
        };
        if room.stock_list.is_empty() {
            user_error("Can't list stock because you're not in a shop.".to_owned())?
        }

        let mut msg = String::new();
        msg.push_str(&format!(
            ansi!("<bold><bgblue><white>| {:40} | {:15} |<reset>\n"),
            ansi!("Item"),
            ansi!("Price")
        ));

        for stock in &room.stock_list {
            if let Some(possession_type) = possession_data().get(&stock.possession_type) {
                let display = &possession_type.display;

                let mut price = stock.list_price;
                if stock.poverty_discount && price > user.credits {
                    price = user.credits;
                }

                msg.push_str(&format!(
                    "| {:40} | {:15.2} |\n",
                    &language::caps_first(&display),
                    &price
                ))
            }
        }
        msg.push_str(ansi!(
            "\nUse <bold>buy<reset> item to purchase something.\n"
        ));
        ctx.trans
            .queue_for_session(&ctx.session, Some(&msg))
            .await?;
        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
