use super::{get_user_or_fail, user_error, UResult, UserVerb, UserVerbRef, VerbContext};
use ansi::ansi;
use async_trait::async_trait;
use uuid::Uuid;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        remaining: &str,
    ) -> UResult<()> {
        if remaining != "abuse" {
            user_error(ansi!("Try <bold>report abuse<reset>.").to_owned())?;
        }

        let user = get_user_or_fail(ctx)?;
        let username = user.username.to_lowercase();
        if ctx.trans.clean_and_count_abusereports(&username).await? > 10 {
            user_error(
                ansi!(
                    "You have too many recent abuse reports logged to record any more. \
                              Contact staff@blastmud.org for help."
                )
                .to_owned(),
            )?;
        }

        let uuid = Uuid::new_v4();
        ctx.trans
            .sendqueue_to_abusereport(&uuid, &username, &ctx.session)
            .await?;
        ctx.trans.queue_for_session(ctx.session, Some(&format!("Up to the last 80 things we sent you since you logged in will now be retained for at least 60 days under reference {}. Send an email to staff@blastmud.org explaining exactly what happened, and include this reference. We'll be able to look it up and investigate.\n", &uuid))).await?;

        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
