use super::{
    get_player_item_or_fail, user_error, UResult, UserError, UserVerb, UserVerbRef, VerbContext,
};
use crate::static_content::room;
use async_trait::async_trait;

pub struct Verb;
#[async_trait]
impl UserVerb for Verb {
    async fn handle(
        self: &Self,
        ctx: &mut VerbContext,
        _verb: &str,
        remaining: &str,
    ) -> UResult<()> {
        let requester = get_player_item_or_fail(ctx).await?;
        let mut resp: String = String::new();
        if remaining == "loc" {
            let (loc_type, loc_code) = requester
                .location
                .split_once("/")
                .ok_or_else(|| UserError("Invalid current location string".to_owned()))?;
            resp.push_str(&format!("Location code: {}\n", loc_code));
            if loc_type == "room" {
                resp.push_str("You're in an ordinary room.\n");
                let room = room::room_map_by_code().get(loc_code).ok_or_else(|| {
                    UserError("Can't find your current room in static data".to_owned())
                })?;
                resp.push_str(&format!(
                    "Grid coords: x={}, y={}, z={}. Coordinates get lower as you go \
                     north, west, down.\n",
                    room.grid_coords.x, room.grid_coords.y, room.grid_coords.z
                ));
            } else if loc_type == "dynroom" {
                resp.push_str("You're in a dynamic room.\n");
            } else {
                resp.push_str(&format!(
                    "Weird - you're in a location of type {}.\n",
                    loc_type
                ));
            }
        } else {
            user_error("Unknown subcommand.".to_owned())?;
        }
        ctx.trans
            .queue_for_session(&ctx.session, Some(&resp))
            .await?;
        Ok(())
    }
}
static VERB_INT: Verb = Verb;
pub static VERB: UserVerbRef = &VERB_INT as UserVerbRef;
