use crate::{
    models::task::TaskDetails,
    regular_tasks::{TaskHandler, TaskRunContext},
    DResult,
};
use async_trait::async_trait;
use std::time;

pub struct ExpireBuffTask;
#[async_trait]
impl TaskHandler for ExpireBuffTask {
    async fn do_task(&self, ctx: &mut TaskRunContext) -> DResult<Option<time::Duration>> {
        let (item_ref, code) = match ctx.task.details {
            TaskDetails::ExpireBuff { ref item, ref code } => Ok((item, code)),
            _ => Err("Bad task type for ExpireBuffTask"),
        }?;

        let (item_type, item_code) = match item_ref.split_once("/") {
            Some(v) => Ok(v),
            None => Err("Invalid item for ExpireBuffTask"),
        }?;

        let item = ctx
            .trans
            .find_item_by_type_code(item_type, item_code)
            .await?;
        let mut item = match item {
            None => return Ok(None),
            Some(i) => (*i).clone(),
        };

        item.temporary_buffs = item
            .temporary_buffs
            .into_iter()
            .filter(|b| &b.code != code)
            .collect();
        ctx.trans.save_item_model(&item).await?;

        Ok(None)
    }
}
pub static EXPIRE_BUFF_TASK: &'static (dyn TaskHandler + Sync + Send) = &ExpireBuffTask;
