use super::{super::room::GridCoords, Dynroom, Dynzone, DynzoneType, Exit, ExitTarget, ExitType};
use crate::models::item::ItemFlag;
use crate::static_content::room::Direction;

pub fn zone() -> Dynzone {
    Dynzone {
        zonetype: DynzoneType::CokMurlApartment,
        zonename: "Condos on King",
        entrypoint_subcode: "doorstep",
        dyn_rooms: vec!(
            ("doorstep", Dynroom {
                subcode: "doorstep",
                name: "Door step",
                short: "DS",
                description: "A sleek hallway, painted white, and lit by futuristic looking bluish-white strip lights that run along the edges of the ceiling. Soft navy blue carpet covers the floor. A beige painted door seems to lead into a private studio apartment",
                exits: vec!(
                    Exit {
                        direction: Direction::WEST,
                        target: ExitTarget::ExitZone,
                        exit_type: ExitType::Doorless
                    },
                    Exit {
                        direction: Direction::EAST,
                        target: ExitTarget::Intrazone { subcode: "studio" },
                        exit_type: ExitType::Doorless
                    }
                ),
                grid_coords: GridCoords { x: 0, y: 0, z: 0 },
                should_caption: true,
                item_flags: vec!(),
                ..Default::default()
            }),
            ("studio", Dynroom {
                subcode: "studio",
                name: "Studio apartment",
                short: "ST",
                description: "An oddly comfortable studio apartment, with worn grey carpet covering the floor. A window to the east has spectacular views of Melbs and the bleak and desolate wasteland beyond it",
                exits: vec!(
                    Exit {
                        direction: Direction::WEST,
                        target: ExitTarget::Intrazone { subcode: "doorstep" },
                        exit_type: ExitType::Doored { description: "a reasonably sturdy looking fire-rated solid core beige painted door" }
                    }
                ),
                grid_coords: GridCoords { x: 1, y: 0, z: 0 },
                should_caption: true,
                has_power: true,
                item_flags: vec!(ItemFlag::DroppedItemsDontExpire,
                                 ItemFlag::PrivatePlace),
                ..Default::default()
            })
        ).into_iter().collect(),
        ..Default::default()
    }
}
