use super::{NPCPronounType, NPCSayInfo, NPCSayType, NPCSpawnPossession, NPC};
use crate::{
    models::{consent::ConsentType, item::LocationActionType},
    static_content::{npc::npc_pronoun_type_to_pronouns, possession_type::PossessionType},
};
use serde::Deserialize;
use serde_yaml::from_str as from_yaml_str;

#[derive(Deserialize)]
struct Citizen {
    code: String,
    name: String,
    spawn_loc: String,
    gender: NPCPronounType,
}

pub fn npc_list() -> Vec<NPC> {
    use NPCSayType::FromFixedList;
    let melbs_citizen_stdsay = NPCSayInfo {
        say_code: "babble",
        frequency_secs: 120,
        talk_type: FromFixedList(vec![
            "I'm so sick of being cloned.",
            "I hope I don't die again today.",
            "I wish the so-called king would do something about the damned zombies everywhere.",
            "I heard in the olden days before the empire babies grew up naturally instead of being taken away to the body factory.",
            "I know people hated the empire, but I kind of wish it was still intact - it was a lot better than what we have now.",
            "I wish there wasn't so much radiation outside of Melbs CBD.",
            "I heard about a guy who went to a special place somewhere around here, and there was a machine that enhanced his wristpad and gave him basically superpowers.",
            "The damn vampire movement... they are all so sneaky, and I never know when they are going to come for my blood.",
        ])
    };

    from_yaml_str::<Vec<Citizen>>(include_str!("melbs_citizen.yaml"))
        .unwrap()
        .into_iter()
        .map(|c|
            NPC {
                code: format!("melbs_citizen_{}", &c.code),
                name: c.name,
                pronouns: npc_pronoun_type_to_pronouns(&c.gender),
                description: "A fairly ordinary looking citizen of Melbs, clearly weary from the harsh reality of post-apocalyptic life".to_owned(),
                spawn_location: format!("room/melbs_{}", &c.spawn_loc),
                spawn_possessions: vec![
                    NPCSpawnPossession {
                        what: PossessionType::Shirt,
                        action_type: LocationActionType::Worn,
                        wear_layer: 0,
                    },
                    NPCSpawnPossession {
                        what: PossessionType::Jeans,
                        action_type: LocationActionType::Worn,
                        wear_layer: 0,
                    },
                ],
                message_handler: None,
                wander_zones: vec!("melbs".to_owned()),
                says: vec!(melbs_citizen_stdsay.clone()),
                player_consents: vec!(ConsentType::Medicine, ConsentType::Share),
                ..Default::default()
            }
        ).collect()
}
