use super::{KillBonus, NPC};
use crate::models::{consent::ConsentType, item::Pronouns};
use crate::static_content::{possession_type::PossessionType, species::SpeciesType};
use serde::Deserialize;
use serde_yaml::from_str as from_yaml_str;

#[derive(Deserialize)]
struct MelbsDog {
    code: String,
    adjectives: String,
    spawn_room: String,
}

pub fn npc_list() -> Vec<NPC> {
    from_yaml_str::<Vec<MelbsDog>>(include_str!("melbs_dog.yaml"))
        .unwrap()
        .into_iter()
        .map(|d|
             NPC {
                 code: format!("melbs_dog_{}", &d.code),
                 name: format!("{} dog", &d.adjectives),
                 pronouns: Pronouns { is_proper: false, ..Pronouns::default_inanimate() },
                 aggression: 12,
                 wander_zones: vec!("melbs".to_owned()),
                 description: "A malnourished looking dog. Its skeleton is visible through its thin and patchy fur. It smells terrible, and certainly doesn't look tame.".to_owned(),
                 aliases: vec!("dog".to_owned()),
                 spawn_location: format!("room/{}", d.spawn_room),
                 intrinsic_weapon: Some(PossessionType::Fangs),
                 species: SpeciesType::Dog,
                 kill_bonus: Some(KillBonus {
                     msg: "On your wristpad: Thank you for helping Melbs with animal control! Here's your fee.",
                     payment: 100,
                 }),
                 player_consents: vec!(ConsentType::Fight),
                 ..Default::default()
             }
        ).collect()
}
