use super::{BenchData, PossessionData, PossessionType};
#[double]
use crate::db::DBTrans;
use crate::{
    message_handler::user_commands::{UResult, UserError},
    models::item::{Item, ItemFlag},
    services::require_power,
    static_content::possession_type::ContainerData,
};
use ansi::ansi;
use async_trait::async_trait;
use mockall_double::double;
use once_cell::sync::OnceCell;

struct NeedPowerBench;
static NEED_POWER_BENCH: NeedPowerBench = NeedPowerBench;

#[async_trait]
impl BenchData for NeedPowerBench {
    async fn check_make(&self, trans: &DBTrans, bench: &Item, _recipe: &Item) -> UResult<()> {
        let (room_type, room_code) = bench
            .location
            .split_once("/")
            .ok_or_else(|| UserError("You're in an invalid location!".to_owned()))?;
        let room = trans
            .find_item_by_type_code(room_type, room_code)
            .await?
            .ok_or_else(|| UserError("Your current room doesn't exist.".to_owned()))?;
        Ok(require_power(&room)?)
    }
}

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| {
        vec![(
            PossessionType::KitchenStove,
            PossessionData {
                display: "kitchen stove",
                aliases: vec!["stove"],
                details: ansi!("A four-element electric stove, with an oven underneath, capable of cooking nutritious meals - or burning a careless amateur cook! [To use, try putting a recipe and the ingredients in it with the <bold>put<reset> command, and turning it on with the <bold>make<reset> command]"),
                weight: 40000,
                container_data: Some(ContainerData {
                    base_weight: 40000,
                    ..Default::default()
                }),
                bench_data: Some(&NEED_POWER_BENCH),
                default_flags: vec![ItemFlag::Bench],
                ..Default::default()
            },
        )]
    })
}
