use super::{PossessionData, PossessionType, WeaponAttackData, WeaponData};
use crate::{models::item::SkillType, static_content::possession_type::DamageType};
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (PossessionType::ButcherKnife,
         PossessionData {
             display: "butcher knife",
             details: "A 30 cm long stainless steel blade, sharp on one edge with a pointy tip. It looks perfect for butchering things, and in a pinch you could probably fight with it too.",
             aliases: vec!("butcher", "knife"),
             weight: 250,
             can_butcher: true,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 0.0,
                 raw_max_to_learn: 2.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} raises {} butcher knife menancingly, preparing to attack {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s butcher knife cuts into {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 2.0,
                     stdev_damage: 2.0,
                     base_damage_type: DamageType::Pierce,
                     other_damage_types: vec!((0.5, DamageType::Slash)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
        (PossessionType::Dagger,
         PossessionData {
             display: "dagger",
             details: "A 30 cm long stainless steel blade, sharp on two edges with a pointy tip, this weapon looks ideal for getting started with bladed close combat.",
             weight: 250,
             can_butcher: true,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 1.0,
                 raw_max_to_learn: 3.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} points {} dagger menancingly, preparing to attack {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s dagger cuts into {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 3.0,
                     stdev_damage: 2.0,
                     base_damage_type: DamageType::Slash,
                     other_damage_types: vec!((0.33334, DamageType::Pierce)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
        (PossessionType::SawtoothMachete,
         PossessionData {
             display: "sawtooth machete",
             details: "A menacing 45 cm long heavy steel blade, with a very sharp serated edge. You could theoretically use this to harvest sugarcane, cutting ruthlessly through the heavy stalks with one hard blow... but it could also be a very formidable weapon in skilled hands.",
             aliases: vec!("machete"),
             weight: 900,
             can_butcher: true,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 2.0,
                 raw_max_to_learn: 4.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} grasps {} machete, preparing to strike {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s machete hits {}'s {} with a cutting blow",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 5.0,
                     stdev_damage: 3.0,
                     base_damage_type: DamageType::Slash,
                     other_damage_types: vec!((0.4, DamageType::Beat)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
        (PossessionType::RusticKatana,
         PossessionData {
             display: "rustic katana",
             details: "A curved 70 cm long steel blade, with a wooden handle wound with cord. It is engraved with 'J. Hannow - Blacksmith. This weapon is clearly a post-apocalyptic creation - it looks a bit, well, rustic, but a big sharp blade of this kind is undoubtedly lethal in the right hands",
             aliases: vec!("katana", "sword"),
             weight: 1000,
             can_butcher: false,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 4.0,
                 raw_max_to_learn: 8.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} raises {} katana menancingly, preparing to attack {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s katana slashes into {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 8.0,
                     stdev_damage: 3.0,
                     base_damage_type: DamageType::Slash,
                     other_damage_types: vec!((0.375, DamageType::Pierce)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
        (PossessionType::Electroblade,
         PossessionData {
             display: "electroblade",
             details: "At first sight, this is an ordinary 70 cm long steel sword, except that it blue sparks arc from its blade, leaving a smell of ozone. This appears to be a weapon that, in skilled hands, will simultaneously slash and electrocute anyone unfortunate to be on the receiving end",
             aliases: vec!("sword", "blade"),
             weight: 1100,
             can_butcher: true,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 7.0,
                 raw_max_to_learn: 13.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} flicks {} electroblade, causing it to spark in anticipation of attacking {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s electroblade zaps {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 13.0,
                     stdev_damage: 7.0,
                     base_damage_type: DamageType::Shock,
                     other_damage_types: vec!((0.231, DamageType::Slash)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
        (PossessionType::FlameScimitar,
         PossessionData {
             display: "flame scimitar",
             details: "A slender curved sword, with the curious property that it glows red hot merely from being swung. In the hands of a blades expert, this would probably through flesh like butter.",
             aliases: vec!("butcher", "knife"),
             weight: 800,
             can_butcher: false,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 12.0,
                 raw_max_to_learn: 14.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} raises {} flame scimitar, swiping it through the air and making it glow red hot, in preparation to attack {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s flame scimitar slices into {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 15.0,
                     stdev_damage: 8.0,
                     base_damage_type: DamageType::Slash,
                     other_damage_types: vec!((0.2, DamageType::Pierce)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
        (PossessionType::NanobladeGladius,
         PossessionData {
             display: "nanoblade gladius",
             details: "It looks at first to be an ordinary sword, only 60 cm long... but one which you quickly realise is actually the legendary nanoblade gladius; a lethal combination of an ancient sword design, with modern pre-fall nanotechnology employed to keep the blade hard and atomically sharp so it can slice through nearly anything, and also inflict terrible damage to any tissue it so much as grazes",
             aliases: vec!("gladius", "sword"),
             weight: 700,
             can_butcher: true,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Blades,
                 raw_min_to_learn: 13.0,
                 raw_max_to_learn: 100.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} raises {} nanoblade gladius menancingly, preparing to attack {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s nanoblade cuts into {}'s {}, activating the horrendous power of the nanites",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 25.0,
                     stdev_damage: 5.0,
                     base_damage_type: DamageType::Slash,
                     other_damage_types: vec!((0.4, DamageType::Pierce)),
                     ..Default::default()
                 },
                 ..Default::default()
             }),
             ..Default::default()
         }),
    ))
}
