use super::{PossessionData, PossessionType, WeaponAttackData, WeaponData};
use crate::{models::item::SkillType, static_content::possession_type::DamageType};
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| {
        vec![(
            PossessionType::SpikedMace,
            PossessionData {
                display: "spiked mace",
                details: "A heavy metal mace with vicious looking spikes",
                aliases: vec!["club", "mace"],
                weight: 3000,
                weapon_data: Some(WeaponData {
                    uses_skill: SkillType::Clubs,
                    raw_min_to_learn: 2.0,
                    raw_max_to_learn: 4.0,
                    normal_attack: WeaponAttackData {
                        start_messages: vec![Box::new(|attacker, victim| {
                            format!(
                                "{} aims {} spiked mace at {}",
                                &attacker.display_for_sentence(1, true),
                                &attacker.pronouns.possessive,
                                &victim.display_for_sentence(1, false),
                            )
                        })],
                        success_messages: vec![Box::new(|attacker, victim, part| {
                            format!(
                                "{}'s spiked mace smashes with force into {}'s {}",
                                &attacker.display_for_sentence(1, true),
                                &victim.display_for_sentence(1, false),
                                &part.display(victim.sex.clone())
                            )
                        })],
                        mean_damage: 3.0,
                        stdev_damage: 3.0,
                        base_damage_type: DamageType::Beat,
                        other_damage_types: vec![(0.25, DamageType::Pierce)],
                        ..Default::default()
                    },
                    ..Default::default()
                }),
                ..Default::default()
            },
        )]
    })
}
