use super::{DamageType, PossessionData, PossessionType, SoakData, WearData};
use crate::static_content::species::BodyPart;
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (
            PossessionType::RustyMetalPot,
            PossessionData {
                display: "rusty metal pot",
                details: "A metal pot that has rusted and is a bit dinged up - it looks like someone that way inclined could wear it as a serviceable helmet.",
                aliases: vec!("pot", "rusted", "rusty"),
                weight: 600,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Head),
                    thickness: 4.0,
                    dodge_penalty: 0.25,
                    soaks: vec!(
                        (DamageType::Beat,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                        (DamageType::Slash,
                         SoakData {
                             min_soak: 1.0,
                             max_soak: 2.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                        (DamageType::Pierce,
                         SoakData {
                             min_soak: 1.0,
                             max_soak: 2.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                    ).into_iter().collect(),
                }),
                ..Default::default()
            }
        ),
        (
            PossessionType::HockeyMask,
            PossessionData {
                display: "hockey mask",
                details: "A white face-hugging fibreglass hockey mask, with small air holes across the face, but no specific hole for the mouth. It looks like it would give a degree of protection to the face, but it might also make someone look like a serial killer!",
                aliases: vec!("mask"),
                weight: 300,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Face),
                    thickness: 4.0,
                    dodge_penalty: 0.25,
                    soaks: vec!(
                        (DamageType::Beat,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                        (DamageType::Slash,
                         SoakData {
                             min_soak: 1.0,
                             max_soak: 2.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                        (DamageType::Pierce,
                         SoakData {
                             min_soak: 1.0,
                             max_soak: 2.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                    ).into_iter().collect()
                }),
                ..Default::default()
            }
        )
    ))
}
