use crate::static_content::possession_type::EatData;

use super::{ChargeData, PossessionData, PossessionType};
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (PossessionType::AnimalSkin,
         PossessionData {
             display: "animal skin",
             aliases: vec!("skin"),
             details: "The skin of an animal of some kind. It looks like you could make something out of this",
             weight: 100,
             ..Default::default()
         }
        ),
        (
            PossessionType::Steak,
            PossessionData {
                display: "steak",
                details: "A hunk of raw red meat, dripping with blood",
                weight: 250,
                eat_data: Some(EatData {
                    hunger_impact: -500,
                    thirst_impact: 0,
                }),
                charge_data: Some(ChargeData {
                    max_charges: 20,
                    charge_name_prefix: "bite",
                    charge_name_suffix: "of food",
                }),
                ..Default::default()
            }
        ),
        (
            PossessionType::SeveredHead,
            PossessionData {
                display: "severed head",
                aliases: vec!("head"),
                details: "A head that has been chopped clean from the body",
                weight: 1500,
                ..Default::default()
            }
        ),
    ))
}
