use std::collections::BTreeMap;

use super::{DamageType, PossessionData, PossessionType, SoakData, WearData};
use crate::static_content::species::BodyPart;
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (
            PossessionType::Shirt,
            PossessionData {
                display: "shirt",
                details: "A fairly plain shirt that probably provides no protection against anything except fashion crimes",
                aliases: vec!("shirt"),
                weight: 70,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Arms,
                                       BodyPart::Chest,
                                       BodyPart::Back),
                    thickness: 2.0,
                    dodge_penalty: 0.0,
                    soaks: BTreeMap::new(),
                }),
                ..Default::default()
            }
        ),
        (
            PossessionType::LeatherJacket,
            PossessionData {
                display: "leather jacket",
                details: "A black leather jacket that looks like it would protect you from falling off a motorbike, or maybe even offer some protection against certain weapons",
                aliases: vec!("jacket"),
                weight: 600,
                wear_data: Some(WearData {
                    covers_parts: vec!(BodyPart::Arms,
                                       BodyPart::Chest,
                                       BodyPart::Back),
                    thickness: 4.0,
                    dodge_penalty: 0.3,
                    soaks: vec!(
                        (DamageType::Beat,
                         SoakData {
                             min_soak: 1.0,
                             max_soak: 2.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                        (DamageType::Slash,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                        (DamageType::Pierce,
                         SoakData {
                             min_soak: 2.0,
                             max_soak: 3.0,
                             damage_probability_per_soak: 0.1
                         }
                        ),
                    ).into_iter().collect(),
                }),
                ..Default::default()
            }
        ),
    ))
}
