use super::{ChargeData, PossessionData, PossessionType, UseData};
use crate::models::{
    consent::ConsentType,
    effect::{Effect, EffectSet, EffectType},
    item::SkillType,
};
use once_cell::sync::OnceCell;

use super::PossessionType::*;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (MediumTraumaKit,
         PossessionData {
             display: "medium trauma kit",
             details: "A collection of bandages and and small gadgets that look like they could, in the right hands, make an unhealthy person healthy again. It looks like when brand new, it could be used 5 times.",
             aliases: vec!("trauma"),
             charge_data: Some(ChargeData {
                 max_charges: 5,
                 charge_name_prefix: "treatment",
                 charge_name_suffix: "worth of supplies",
                 ..Default::default()
             }),
             use_data: Some(UseData {
                 uses_skill: SkillType::Medic,
                 diff_level: 10.0,
                 crit_fail_effects: Some(EffectSet {
                     effect_type: EffectType::Ephemeral,
                     effects: vec!(
                         Effect::BroadcastMessage {
                             delay_secs: 0,
                             messagef: Box::new(|player, _item, target|
                                 format!(
                                     "{} attempts to heal {} with a trauma kit, but fucks it up badly\n",
                                     &player.display_for_sentence(1, true),
                                     &if target.item_type == player.item_type && target.item_code == player.item_code {
                                         player.pronouns.intensive.clone()
                                     } else {
                                         target.display_for_sentence(1, false)
                                     }
                                 ),
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 0, base_effect: -2, skill_multiplier: -3.0,
                             max_effect: -5,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "Fuck! The trauma kit makes {}'s condition worse",
                                     target.display_for_sentence(1, false))
                             )
                         }
                     )
                 }),
                 fail_effects: Some(EffectSet {
                     effect_type: EffectType::Ephemeral,
                     effects: vec!(
                         Effect::BroadcastMessage {
                             delay_secs: 0,
                             messagef: Box::new(|player, _item, target|
                               format!(
                                   "{} attempts unsuccessfully to heal {} with a trauma kit\n",
                                   &player.display_for_sentence(1, true),
                                   &if target.item_type == player.item_type && target.item_code == player.item_code {
                                       player.pronouns.intensive.clone()
                                   } else {
                                       target.display_for_sentence(1, false)
                                   }
                               )
                             )
                       },
                     )
                 }),
                 success_effects: Some(EffectSet {
                     effect_type: EffectType::Bandages,
                     effects: vec!(
                         Effect::BroadcastMessage {
                             delay_secs: 0,
                             messagef: Box::new(|player, _item, target|
                                 format!(
                                     "{} expertly heals {} with a trauma kit\n",
                                     &player.display_for_sentence(1, true),
                                     &if target.item_type == player.item_type && target.item_code == player.item_code {
                                         player.pronouns.intensive.clone()
                                     } else {
                                         target.display_for_sentence(1, false)
                                     }
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 0, base_effect: 2, skill_multiplier: 8.0,
                             max_effect: 10,
                             message: Box::new(
                               |target|
                                 format!(
                                     "FUUUCK! It hurts {}, but also starts to soothe {}",
                                     target.display_for_sentence(1, false),
                                     &target.pronouns.object
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 10, base_effect: 2, skill_multiplier: 7.0,
                             max_effect: 9,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "FUUUCK! It hurts {}, but also starts to soothe {}",
                                     target.display_for_sentence(1, false),
                                     &target.pronouns.object
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 20, base_effect: 1, skill_multiplier: 6.0,
                             max_effect: 7,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "The bandages soothe {}'s wounds",
                                     target.display_for_sentence(1, false),
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 30, base_effect: 1, skill_multiplier: 5.0,
                             max_effect: 6,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "The bandages soothe {}'s wounds",
                                     target.display_for_sentence(1, false),
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 40, base_effect: 0, skill_multiplier: 4.0,
                             max_effect: 4,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "The bandages soothe {}'s wounds",
                                     target.display_for_sentence(1, false),
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 50, base_effect: 0, skill_multiplier: 3.0,
                             max_effect: 3,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "The bandages soothe {}'s wounds",
                                     target.display_for_sentence(1, false),
                                 )
                             )
                         },
                         Effect::ChangeTargetHealth {
                             delay_secs: 60, base_effect: 0, skill_multiplier: 2.0,
                             max_effect: 2,
                             message: Box::new(
                                 |target|
                                 format!(
                                     "The bandages soothe {}'s wounds",
                                     target.display_for_sentence(1, false),
                                 )
                             )
                         },
                         Effect::BroadcastMessage {
                             delay_secs: 60,
                             messagef: Box::new(|_player, _item, target|
                                 format!(
                                     "The bandages wrapping {} crumble and fall away, their healing capabilities fully expended.\n",
                                     target.display_for_sentence(1, false)
                                 )
                             )
                         }
                     ),
                 }),
                 errorf: Box::new(
                     |item, target|
                     if target.death_data.is_some() {
                         Some(format!("It is too late, {}'s dead", target.pronouns.subject))
                     } else if target.item_type != "player" && target.item_type != "npc" {
                         Some("It only works on animals.".to_owned())
                     } else if target.active_effects.iter().any(|e| e.0 == EffectType::Bandages) {
                         Some(format!(
                             "You see no reason to use {} on {}",
                             item.display_for_sentence(1, false),
                             target.display_for_sentence(1, false)
                         ))
                     } else {
                         None
                     }),
                 needs_consent_check: Some(ConsentType::Medicine),
                 ..Default::default()
             }),
             becomes_on_spent: Some(EmptyMedicalBox),
             ..Default::default()
         }
        ),
        (
            EmptyMedicalBox,
            PossessionData {
                display: "empty medical box",
                details: "An empty box that looks like it once had something medical in it.",
                aliases: vec!("box"),
                ..Default::default()
            }
        )
    ))
}
