use super::{DamageType, PossessionData, PossessionType, WeaponAttackData, WeaponData};
use crate::models::{effect::EffectType, item::SkillType};
use once_cell::sync::OnceCell;

pub fn data() -> &'static Vec<(PossessionType, PossessionData)> {
    static D: OnceCell<Vec<(PossessionType, PossessionData)>> = OnceCell::new();
    &D.get_or_init(|| vec!(
        (PossessionType::AntennaWhip,
         PossessionData {
             display: "antenna whip",
             details: "A crudely fashioned whip made from a broken metal antenna. It looks a bit flimsy, but it \
                       might do you until you get a better weapon!",
             aliases: vec!("whip"),
             weight: 1000,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Whips,
                 raw_min_to_learn: 0.0,
                 raw_max_to_learn: 2.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} lines up {} antenna whip for a strike on {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s antenna whip scores a painful red line across {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 3.0,
                     stdev_damage: 3.0,
                     ..Default::default()
                 },
                 power_attack: Some(WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} rears back {} antenna whip for a power attack on {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s antenna whip hits {}'s {} with great force",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 9.0,
                     stdev_damage: 6.0,
                     crit_effects: vec![(0.05, EffectType::Bleed)],
                     ..Default::default()
                 }),
                 ..Default::default()
             }),
             ..Default::default()
         }
        ),
        (PossessionType::LeatherWhip,
         PossessionData {
             display: "leather whip",
             details: "A whip made from stitched together animal skins... it looks like a formidable weapon, and in the right hands will make someone look like Indiana Jones!",
             aliases: vec!("whip"),
             weight: 800,
             weapon_data: Some(WeaponData {
                 uses_skill: SkillType::Whips,
                 raw_min_to_learn: 0.0,
                 raw_max_to_learn: 2.0,
                 normal_attack: WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} lines up {} leather whip for a strike on {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s leather whip scores a painful red line across {}'s {}",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 4.0,
                     stdev_damage: 2.0,
                     base_damage_type: DamageType::Beat,
                     crit_effects: vec![(0.05, EffectType::Bleed)],
                     other_damage_types: vec!((0.25, DamageType::Slash)),
                     ..Default::default()
                 },
                 power_attack: Some(WeaponAttackData {
                     start_messages: vec!(
                         Box::new(|attacker, victim|
                                  format!("{} rears back {} leather whip for a power strike on {}",
                                          &attacker.display_for_sentence(1, true),
                                          &attacker.pronouns.possessive,
                                          &victim.display_for_sentence(1, false),
                                  )
                         )
                     ),
                     success_messages: vec!(
                         Box::new(|attacker, victim, part|
                                  format!("{}'s leather whip hits {}'s {} with great force",
                                          &attacker.display_for_sentence(1, true),
                                          &victim.display_for_sentence(1, false),
                                          &part.display(victim.sex.clone())
                                  )
                         )
                     ),
                     mean_damage: 12.0,
                     stdev_damage: 4.0,
                     base_damage_type: DamageType::Beat,
                     crit_effects: vec![(0.3, EffectType::Bleed)],
                     other_damage_types: vec!((0.25, DamageType::Slash)),
                     ..Default::default()
                 }),
                 ..Default::default()
             }),
             ..Default::default()
         }
        ),
    ))
}
