use crate::{
    models::item::LocationActionType,
    static_content::{
        fixed_item::FixedItem,
        possession_type::{possession_data, PossessionData, PossessionType},
    },
};

use super::{Direction, Room, SimpleRoom};
use serde_yaml::from_str as from_yaml_str;

pub fn room_list() -> Vec<Room> {
    from_yaml_str::<Vec<SimpleRoom<()>>>(include_str!("computer_museum.yaml"))
        .unwrap()
        .into_iter()
        .map(|r| r.into())
        .collect()
}

pub fn fixed_items() -> Vec<FixedItem> {
    vec![FixedItem {
        code: "computer_museum_club_door_lock".to_owned(),
        name: "basic keyed lock".to_owned(),
        description: "A basic lock that looks like it needs a key to open".to_owned(),
        location: "room/computer_museum_hackers_club".to_owned(),
        proper_noun: false,
        aliases: vec!["lock".to_owned()],
        action_type: LocationActionType::InstalledOnDoorAsLock(Direction::SOUTH),
        ..Default::default()
    }]
}

pub fn fixed_item_properties() -> Vec<(&'static str, PossessionData)> {
    let lock_tmpl = possession_data().get(&PossessionType::Basiclock).unwrap();
    vec![(
        "computer_museum_club_door_lock",
        PossessionData {
            lockcheck_handler: lock_tmpl.lockcheck_handler,
            ..Default::default()
        },
    )]
}
